module Regexifi
  module Email
    # WARNING:
    # If you want to be absolutely true to the spec, most EMAIL RFC are almost impossible to define using Regex alone.
    # Or it becomes clunky and unwieldy. Here are the most common REGEXP , that are commonly used in the "wild".
    # If you want a better way to parse an email - please refer to this discussion.
    # https://stackoverflow.com/questions/22993545/ruby-email-validation-with-regex

    # Used by a popular authentication gem Devise
    DEVISE = /^([\w\.%\+\-]+)@([\w\-]+\.)+([\w]{2,})$/i.freeze

    # Based on the Devise Regexp above, but actually UTF-8 / IDN friendly
    # If you need to pass internationalized domains (IDN) - https://en.wikipedia.org/wiki/Internationalized_domain_name
    # that use Cyrillic or Asian characters for example, this might be a better choice.
    IDN = /^([[[:word:]]\.%\+\-]+)@([[[:word:]]\-]+\.)+([[:alpha:]]{2,})$/i.freeze

    # This is a commonly seen in Rails apps regex from Michael's Hartl's tutorial
    # IT's similar to DEVISE's default Regex with slight modification ( not allowing 1-letter TLDs)
    # Original = /\A[\w+\-.]+@[a-z\d\-]+(\.[a-z\d\-]+)*\.[a-z]+\z/ix.freeze
    RAILS_TUTORIAL = /\A[\w+\-.]+@[a-z\d\-]+(\.[a-z\d\-]+)*\.[a-z]{2,}+\z/i.freeze

    # This is a standard developed by WHATWG - Web Hypertext Application Technology Working Group based on the RFC1123
    # standard that defines rules for domains.
    # https://html.spec.whatwg.org/multipage/input.html#valid-e-mail-address
    WHATWG = %r{^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+
    @[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?
    (?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*$}ix.freeze

    # RFC1123 = /\A(?<domain>(?!.{254,})((((([A-Za-z0-9])([A-Za-z0-9\-]){0,62}\.)*([A-Za-z])
    # ([A-Za-z0-9\-]){0,62}))|((?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}
    # (?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?))))\Z/ix
    RFC5322 = %r{\A(?<local>(([A-Za-z0-9!#\$%&'*\+\-/=\?\^_`\{\}\|~])+)(\.([A-Za-z0-9!#\$%&'*\+\-/=\?\^_`\{\}\|~])+)*)
    @((?<domain>(?!.{254,})((((([A-Za-z0-9])([A-Za-z0-9\-]){0,62}\.)*([A-Za-z])([A-Za-z0-9\-]){0,62}))|
    ((?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)))))\Z}ix.freeze

    # RFC6068 defines the whole URI spec, but for out purposes here we just need email address REGEXP,
    # which currently is defined as this
    # /\A[a-zA-Z0-9.!\#$%&'*+\/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?
    # (?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*\z/ix
    # However if you don't have a compelling reason to spell it out, it is recommended to use REGEX defined in Ruby's
    # standard library
    RFC6068 = URI::MailTo::EMAIL_REGEXP
  end
end
